/* HEADER */
import com.maverick.ssh.*;
import com.maverick.ssh1.Ssh1Client;
import com.sshtools.net.*;

import java.io.*;
import com.sshtools.publickey.ConsoleKnownHostsKeyVerification;
/**
 * This example demonstrates the latest feature addition to the API. The use
 * of an expect type session to control a remote process.
 *
 * @author Lee David Painter
 */
public class ExpectShell {

	public static void main(String[] args) {


		final BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));

		try {

                    System.out.print("Hostname: ");
                    String hostname = reader.readLine();

                    int idx = hostname.indexOf(':');
                    int port = 22;
                    if (idx > -1) {
                        port = Integer.parseInt(hostname.substring(idx + 1));
                        hostname = hostname.substring(0, idx);

                    }

                    System.out.print("Username [Enter for " + System.getProperty("user.name") +
                                     "]: ");
                    String username = reader.readLine();

                    if (username == null || username.trim().equals(""))
                        username = System.getProperty("user.name");

                    System.out.println("Connecting to " + hostname);

                    /**
                     * Create an SshConnector instance
                     */
                    SshConnector con = SshConnector.getInstance();
                    con.setKnownHosts(new ConsoleKnownHostsKeyVerification());

                    /**
                     * Connect to the host, we need a buffered connection for
                     * the Shell class
                     */
                    final SshClient ssh = con.connect(new SocketTransport(hostname, port),
                                                      username,
                                                      true);

                    /**
                     * Determine the version
                     */
                    if (ssh instanceof Ssh1Client)
                        System.out.println(hostname + " is an SSH1 server");
                    else
                        System.out.println(hostname + " is an SSH2 server");

                    /**
                     * Authenticate the user using password authentication
                     */
                    com.maverick.ssh.PasswordAuthentication pwd = new com.maverick.ssh.
                                                                  PasswordAuthentication();

                    do {
                        System.out.print("Password: ");
                        pwd.setPassword(reader.readLine());
                    } while (ssh.authenticate(pwd) != SshAuthentication.COMPLETE
                             && ssh.isConnected());

                    /**
                     * Start a session and do basic IO
                     */
                    if (ssh.isAuthenticated()) {

                        Shell shell = new Shell(ssh);

                        // If auto detection does not work you can set the required properties here

                        shell.createSession();

                        if(shell.getEnvironment().hasEnvironmentVariable("HOSTNAME"))
                            System.out.println("We are connected to " + shell.getEnvironment().getEnvironmentVariable("HOSTNAME"));

                        System.out.println("Remote operating system is " + shell.getEnvironment().getOperatingSystem());

                        ShellProcess process;
                        boolean isWindows = shell.getEnvironment().getOSType()==ShellEnvironment.OS_WINDOWS;

                        if(shell.getEnvironment().hasEnvironmentVariable("USERPROFILE")) {
                            System.out.println("User home is " + shell.getEnvironment().getEnvironmentVariable("USERPROFILE"));

                        } else if(shell.getEnvironment().hasEnvironmentVariable("HOME")) {
                            System.out.println("User home is " + shell.getEnvironment().getEnvironmentVariable("HOME"));

                        }

                        if(isWindows) {
                           process = shell.execute("dir");
                        } else {
                            process = shell.execute("ls --color=never");
                        }

                        String line;
                        while((line = process.readLine()) != null) {
                            System.out.println(line);
                        }


                        if(!isWindows) {

                            String passwordPattern;
                            String completedPattern;

                            switch(shell.getEnvironment().getOSType()) {
                               case ShellEnvironment.OS_SOLARIS:
                                   passwordPattern = "Password:";
                                   completedPattern = "successfully";
                                   break;
                               default:
                                   passwordPattern = "password:";
                                   completedPattern = "successfully";
                                   break;
                            }
                            // Now that the process has returned a null line
                            // we have exhausted all its output

                            // Automate the changing of the users password. Note the
                            // use of Thread.sleep(200) to create a pause between the
                            // receiving of the password prompt and sending of the
                            // password.
                            //
                            // We pause because the passwd command may reset the
                            // input buffer, and sending the password too quickly
                            // may cause some or all of the password to be lost
                            // or echo'd back to the screen
                            process = shell.execute("passwd lee");

                            try {
                                process.expect(passwordPattern, 10000, true);
                                Thread.sleep(200);
                                process.typeAndReturn("123defs");

                                process.expect(passwordPattern, 10000, true);
                                Thread.sleep(200);
                                process.typeAndReturn("123defs");

                                process.expect(completedPattern, 10000, true);

                            } catch (ShellTimeoutException ex) {
                                System.out.println(
                                        "Expect operation timed out. Sending interrupt to kill the process");
                                process.interrupt();
                            } finally {
                                process.close();
                                System.out.print(process.toString());
                            }
                        }

                        shell.exit();
                        System.out.println("Shell has exited");


                    }

                    ssh.disconnect();
                } catch(Throwable t) {
                    t.printStackTrace();
                }
	}

}

